#include "MESH_Transform.h"

namespace X3DTK {
namespace MESH {

Transform::Transform()
: X3DGroupingNode(), _center(SFVec3f(0.0f, 0.0f, 0.0f)), _rotation(SFRotation(0.0f, 0.0f, 1.0f, 0.0f)), _scale(SFVec3f(1.0f, 1.0f, 1.0f)), _scaleOrientation(SFRotation(0.0f, 0.0f, 1.0f, 0.0f)), _translation(SFVec3f(0.0f, 0.0f, 0.0f))
{
  defineTypeName("Transform");
}

Transform::Transform(const SFVec3f &center, const SFRotation &rotation, const SFVec3f &scale,  const SFRotation &scaleOrientation,  const SFVec3f &translation)
: X3DGroupingNode(), _center(center), _rotation(rotation), _scale(scale), _scaleOrientation(scaleOrientation), _translation(translation)
{
  defineTypeName("Transform");
}

Transform::Transform(const Transform &T)
: X3DGroupingNode(T), _center(T._center), _rotation(T._rotation), _scale(T._scale), _scaleOrientation(T._scaleOrientation), _translation(T._translation)
{
}

SFAbstractNode Transform::clone() const
{
  return new Transform(*this);
}

Transform::~Transform()
{
}

void Transform::setCenter(const SFVec3f &center)
{
  _center = center;
}

void Transform::setRotation(const SFRotation &rotation)
{
  _rotation = rotation;
}

void Transform::setScale(const SFVec3f &scale)
{
  _scale = scale;
}

void Transform::setScaleOrientation(const SFRotation &scaleOrientation)
{
  _scaleOrientation = scaleOrientation;
}

void Transform::setTranslation(const SFVec3f &translation)
{
  _translation = translation;
}

SFMatrix34f Transform::getTransform() const 
{
  return SFMatrix34f(_translation, _center, _rotation, _scaleOrientation, _scale);
}

}
}
