#include "X3D_SceneSaver.h"
#include "DFSGraphTraversal.h"
#include "X3D_SceneSaverGetCoreVisitor.h"
#include "X3D_SceneSaverSetCoreVisitor.h"
#include "X3D_SceneSaverWriteCoreVisitor.h"

#include <fstream>
#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

SceneSaver::SceneSaver()
: X3DSceneSaver()
{
  stateVariables = Singleton<SceneSaverStateVariables>::getInstance();
  
  graphTraversalMap["Get"] = new DFSGraphTraversal();  
  graphTraversalMap["Get"]->setComponentVisitor(new SceneSaverGetCoreVisitor());

  graphTraversalMap["Set"] = new DFSGraphTraversal();  
  graphTraversalMap["Set"]->setComponentVisitor(new SceneSaverSetCoreVisitor());
  
  graphTraversalMap["Write"] = new DFSGraphTraversal();  
  graphTraversalMap["Write"]->setComponentVisitor(new SceneSaverWriteCoreVisitor());
}

SceneSaver::~SceneSaver()
{
  Singleton<SceneSaverStateVariables>::removeInstance();
  
  delete graphTraversalMap["Write"];
}

void SceneSaver::saveAs(Scene *S, const SFString &fileName)
{
  if (S == 0)
  {
    cx3d << "unable to write to file, because the Scene is NULL!" << endl;
    return;
  }
    
  stateVariables->init();
  graphTraversalMap["Get"]->traverse(S);
  graphTraversalMap["Set"]->traverse(S);
  stateVariables->initNodes();
  
  ofstream output(fileName);
  
  output << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" << endl;
  output << "<X3D>" << endl;
  output << "  <head>" << endl; 
  std::list<std::pair<SFString, SFString> > history = S->getProcessHistory().getProcessMap();
  for (std::list<std::pair<SFString, SFString> >::const_iterator h = history.begin(); h != history.end(); ++h)  
    output << "    <meta name=\"process\" content=\"" + (*h).first + "\" date=\"" + (*h).second + "\"/>" << endl;
  output << "  </head>" << endl; 
  
  graphTraversalMap["Write"]->traverse(S);
  MFString text = stateVariables->getOutput();
  
  for (MFString::const_iterator l = text.begin(); l != text.end(); ++l)
    output << (*l) << endl;

  output << "</X3D>" << endl;
  stateVariables->finish();    
}

void SceneSaver::save(Scene *S)
{
  saveAs(S, S->getFileName());
}

}
}
